# -*- coding: utf-8 -*-

# FOGLAMP_BEGIN
# See: http://foglamp.dianomic.com
# FOGLAMP_END


__author__ = "Mohit Rajput"
__copyright__ = "Copyright (c) 2024 Dianomic Systems"
__license__ = "Apache 2.0"
__version__ = "${VERSION}"


import os
import linecache
import logging
import time
import tracemalloc
from threading import Timer

LOG_LEVEL = {
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
    "WARN": logging.WARN,
    "ERROR": logging.ERROR
}

try:
    from foglamp.common import logger
    _LOGGER = logger.setup(__name__, level=logging.INFO)
except:
    class _LOGGER:
        debug = print
        info = print
        warn = print
        error = print

DEFAULT_SCORE = -1
DEFAULT_FORECAST = -1


class TrackMemoryAllocation:
    """Track the memory allocated during code execution.

    References:
        https://docs.python.org/3/library/tracemalloc.html#functions
        https://docs.python.org/3/library/tracemalloc.html#pretty-top
        https://stackoverflow.com/questions/552744/how-do-i-profile-memory-usage-in-python
    """

    def __init__(self):
        self.memory_retained_at_end = None
        self.peak_memory_consumption = None

    def __enter__(self):
        tracemalloc.start()
        tracemalloc.clear_traces()

        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        self.memory_retained_at_end, self.peak_memory_consumption = (
            tracemalloc.get_traced_memory()
        )
        tracemalloc.stop()

    def stats(self):
        """Return observed peak allocated memory and retained memory by code snippet."""
        if self.memory_retained_at_end is not None:
            self.memory_retained_at_end = self.memory_retained_at_end / 1024 / 1024
            self.peak_memory_consumption = self.peak_memory_consumption / 1024 / 1024

        return self.memory_retained_at_end, self.peak_memory_consumption

class RepeatedTimer(object):
    """Class that allows execution at fixed timed interval."""

    def __init__(self, interval, function, *args, **kwargs):
        self._timer = None
        self.interval = interval
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.is_running = False
        self.start()
        self.start_time = time.time()
        self.current_time = time.time()

    def _run(self):
        self.is_running = False
        self.start()
        self.function(*self.args, **self.kwargs)

    def start(self):
        if not self.is_running:
            self._timer = Timer(self.interval, self._run)
            self._timer.start()
            self.is_running = True
            self.current_time = time.time()

    def time_passed(self):
        return self.current_time - self.start_time

    def reset_timer(self):
        self.start_time = time.time()
        self.current_time = time.time()

    def stop(self):
        self._timer.cancel()
        self.is_running = False


class TriggerExecution:
    """A class that help repeatedly indicate if execution is allowed when the configured time has expired."""

    def __init__(self, wait_period=10, timer_refresh_interval=0.01):
        self.wait_period = wait_period
        self.repeated_timer = RepeatedTimer(timer_refresh_interval, TriggerExecution._execution_allowed, self)
        self.execution_allowed = False

    def _execution_allowed(self):
        if self.repeated_timer.time_passed() >= self.wait_period:
            self.execution_allowed = True
        else:
            self.execution_allowed = False

    @property
    def is_execution_allowed(self):
        return self.execution_allowed

    @property
    def reset_timer(self):
        self.repeated_timer.reset_timer()
        self.execution_allowed = False

    @property
    def shutdown_timer(self):
        self.repeated_timer.stop()
        self.repeated_timer = None
